@@ -30,6 +30,8 @@ include_files = [
         os.path.join('lib', 'gtk-3.0')),
     (os.path.join(sys.prefix, 'lib', 'gdk-pixbuf-2.0'),
         os.path.join('lib', 'gdk-pixbuf-2.0')),
+    (os.path.join(sys.prefix, 'lib', 'evince'),
+        os.path.join('lib', 'evince')),
     (os.path.join(sys.prefix, 'share', 'locale'),
         os.path.join('share', 'locale')),
     (os.path.join(sys.prefix, 'share', 'icons', 'Adwaita'),
@@ -42,6 +44,8 @@ include_files = [
 
 required_gi_namespaces = [
     'Atk-1.0',
+    'EvinceDocument-3.0',
+    'EvinceView-3.0',
     'GLib-2.0',
     'GModule-2.0',
     'GObject-2.0',
@@ -67,8 +71,9 @@ def replace_path(match):
     if sys.platform == 'darwin':
         libs = [os.path.join('@executable_path', l) for l in libs]
     return 'shared-library="%s"' % ','.join(libs)
-lib_re = re.compile(r'shared-library="([^\"]*)"')
 
+
+lib_re = re.compile(r'shared-library="([^\"]*)"')
 required_libs = set()
 temp = tempfile.mkdtemp()
 for ns in required_gi_namespaces:
@@ -91,7 +96,6 @@ for ns in required_gi_namespaces:
 
 if sys.platform == 'win32':
     required_libs.update([
-        'libcroco-0.6-3.dll',
         'libepoxy-0.dll',
         ])
     lib_path = os.getenv('PATH', os.defpath).split(os.pathsep)
