@@ -4,8 +4,8 @@ import gettext
 import webbrowser
 
 import tryton.rpc as rpc
-from tryton.common import RPCProgress, RPCExecute, RPCException
-from tryton.common import message, selection, file_write, file_open, mailto
+from tryton.common import RPCExecute, RPCException
+from tryton.common import message, selection, file_write, file_open
 from tryton.config import CONFIG
 from tryton.pyson import PYSONDecoder
 
@@ -15,56 +15,43 @@ _ = gettext.gettext
 class Action(object):
 
     @staticmethod
-    def exec_report(name, data, direct_print=False, email_print=False,
-            email=None, context=None):
+    def exec_report(name, data, direct_print=False, context=None):
         if context is None:
             context = {}
-        if email is None:
-            email = {}
-        data = data.copy()
-        ctx = rpc.CONTEXT.copy()
-        ctx.update(context)
-        ctx['direct_print'] = direct_print
-        ctx['email_print'] = email_print
-        ctx['email'] = email
-        args = ('report', name, 'execute', data.get('ids', []), data, ctx)
-        try:
-            res = RPCProgress('execute', args).run()
-        except RPCException:
-            return False
-        if not res:
-            return False
-        (type, data, print_p, name) = res
-        if not print_p and direct_print:
-            print_p = True
-
-        fp_name = file_write((name, type), data)
-        if email_print:
-            mailto(to=email.get('to'), cc=email.get('cc'),
-                subject=email.get('subject'), body=email.get('body'),
-                attachment=fp_name)
         else:
+            context = context.copy()
+        context['direct_print'] = direct_print
+        ids = data.get('ids', [])
+
+        def callback(result):
+            try:
+                result = result()
+            except RPCException:
+                return
+            type, data, print_p, name = result
+            if not print_p and direct_print:
+                print_p = True
+            fp_name = file_write((name, type), data)
             file_open(fp_name, type, print_p=print_p)
-        return True
+        RPCExecute(
+            'report', name, 'execute', ids, data, context=context,
+            callback=callback)
 
     @staticmethod
-    def execute(act_id, data, action_type=None, context=None, keyword=False):
-        # Must be executed synchronously to avoid double execution
-        # on double click.
-        if not action_type:
-            action, = RPCExecute('model', 'ir.action', 'read', [act_id],
-                ['type'], context=context)
-            action_type = action['type']
-        action, = RPCExecute('model', action_type, 'search_read',
-            [('action', '=', act_id)], 0, 1, None, None,
-            context=context)
+    def execute(action, data, context=None, keyword=False):
+        if isinstance(action, int):
+            # Must be executed synchronously to avoid double execution
+            # on double click.
+            action = RPCExecute(
+                'model', 'ir.action', 'get_action_value', action,
+                context=context)
         if keyword:
             keywords = {
                 'ir.action.report': 'form_report',
                 'ir.action.wizard': 'form_action',
                 'ir.action.act_window': 'form_relate',
                 }
-            action.setdefault('keyword', keywords.get(action_type, ''))
+            action.setdefault('keyword', keywords.get(action['type'], ''))
         Action._exec_action(action, data, context=context)
 
     @staticmethod
@@ -89,12 +76,15 @@ class Action(object):
             if not data.get('ids') or not data.get('model'):
                 return name
             max_records = 5
+            ids = list(filter(lambda id: id >= 0, data['ids']))[:max_records]
+            if not ids:
+                return name
             rec_names = RPCExecute('model', data['model'],
-                'read', data['ids'][:max_records], ['rec_name'],
+                'read', ids, ['rec_name'],
                 context=context)
             name_suffix = _(', ').join([x['rec_name'] for x in rec_names])
-            if len(data['ids']) > max_records:
-                name_suffix += _(',\u2026')
+            if len(data['ids']) > len(ids):
+                name_suffix += _(',...')
             return _('%s (%s)') % (name, name_suffix)
 
         data['action_id'] = action['id']
@@ -120,7 +110,6 @@ class Action(object):
             action_ctx.update(
                 decoder.decode(action.get('pyson_context') or '{}'))
             ctx.update(action_ctx)
-            ctx.update(context)
 
             ctx['context'] = ctx
             decoder = PYSONDecoder(ctx)
@@ -160,16 +149,15 @@ class Action(object):
                 name = add_name_suffix(name, context)
             Window.create_wizard(action['wiz_name'], data,
                 direct_print=action.get('direct_print', False),
-                email_print=action.get('email_print', False),
-                email=action.get('email'), name=name,
+                name=name,
                 context=context, icon=(action.get('icon.rec_name') or ''),
                 window=action.get('window', False))
 
         elif action['type'] == 'ir.action.report':
-            Action.exec_report(action['report_name'], data,
-                    direct_print=action.get('direct_print', False),
-                    email_print=action.get('email_print', False),
-                    email=action.get('email'), context=context)
+            Action.exec_report(
+                action['report_name'], data,
+                direct_print=action.get('direct_print', False),
+                context=context)
 
         elif action['type'] == 'ir.action.url':
             if action['url']:
@@ -188,29 +176,13 @@ class Action(object):
 
         keyact = {}
         for action in actions:
-            keyact[action['name'].replace('_', '')] = action
+            keyact[action['name'].split(' / ')[-1]] = action
 
         res = selection(_('Select your action'), keyact, alwaysask=alwaysask)
         if res:
             (name, action) = res
-            Action._exec_action(action, data, context=context)
+            Action.execute(action, data, context=context)
             return (name, action)
         elif not len(keyact) and warning:
             message(_('No action defined.'))
         return False
-
-    @staticmethod
-    def evaluate(action, atype, record):
-        '''
-        Evaluate the action with the record.
-        '''
-        action = action.copy()
-        email = {}
-        if 'pyson_email' in action:
-            email = record.expr_eval(action['pyson_email'])
-            if not email:
-                email = {}
-        if 'subject' not in email:
-            email['subject'] = action['name'].replace('_', '')
-        action['email'] = email
-        return action
