@@ -223,8 +223,29 @@ class SelectionField(Field):
 
     _default = None
 
-    def get_client(self, record):
-        return record.value.get(self.name)
+
+class MultiSelectionField(Field):
+
+    _default = None
+
+    def get(self, record):
+        value = super().get(record)
+        if not value:
+            value = self._default
+        else:
+            value.sort()
+        return value
+
+    def get_eval(self, record):
+        value = super().get_eval(record)
+        if value is None:
+            value = []
+        return value
+
+    def set_client(self, record, value, force_change=False):
+        if value:
+            value = sorted(value)
+        super().set_client(record, value, force_change=force_change)
 
 
 class DateTimeField(Field):
@@ -334,6 +355,28 @@ class FloatField(Field):
         shift = int(round(math.log(abs(factor), 10)))
         return (digits[0] + shift, digits[1] - shift)
 
+    def get_symbol(self, record, symbol):
+        if record and symbol in record.group.fields:
+            value = self.get(record) or 0
+            sign = 1
+            if value < 0:
+                sign = -1
+            elif value == 0:
+                sign = 0
+            symbol_field = record.group.fields[symbol]
+            symbol_name = symbol_field.attrs.get('relation')
+            symbol_id = symbol_field.get(record)
+            if symbol_name and symbol_id is not None and symbol_id >= 0:
+                try:
+                    return RPCExecute(
+                        'model', symbol_name, 'get_symbol', symbol_id, sign,
+                        context=record.get_context())
+                except RPCException:
+                    logger.warn(
+                        "Fail to fetch symbol for %s,%s",
+                        symbol_name, symbol_id)
+        return '', 1
+
     def convert(self, value):
         try:
             return locale.atof(value)
@@ -356,14 +399,14 @@ class FloatField(Field):
         value = record.value.get(self.name)
         if value is not None:
             digits = self.digits(record, factor=factor)
+            d = value * factor
+            if not isinstance(d, Decimal):
+                d = Decimal(repr(d))
             if digits:
                 p = int(digits[1])
             else:
-                d = value * factor
-                if not isinstance(d, Decimal):
-                    d = Decimal(repr(d))
                 p = -int(d.as_tuple().exponent)
-            return locale.format('%.*f', (p, value * factor), True)
+            return locale.localize('{0:.{1}f}'.format(d, p), True)
         else:
             return ''
 
@@ -372,7 +415,7 @@ class NumericField(FloatField):
 
     def convert(self, value):
         try:
-            return locale.atof(value, Decimal)
+            return Decimal(locale.delocalize(value))
         except decimal.InvalidOperation:
             return self._default
 
@@ -428,26 +471,27 @@ class M2OField(Field):
         return self.get(record) is None
 
     def get_client(self, record):
-        rec_name = record.value.get(self.name + '.rec_name')
+        rec_name = record.value.get(self.name + '.', {}).get('rec_name')
         if rec_name is None:
             self.set(record, self.get(record))
-            rec_name = record.value.get(self.name + '.rec_name') or ''
-        return rec_name
+            rec_name = record.value.get(self.name + '.', {}).get('rec_name')
+        return rec_name or ''
 
     def set_client(self, record, value, force_change=False):
         if isinstance(value, (tuple, list)):
             value, rec_name = value
         else:
             if value == self.get(record):
-                rec_name = record.value.get(self.name + '.rec_name', '')
+                rec_name = record.value.get(
+                    self.name + '.', {}).get('rec_name', '')
             else:
                 rec_name = ''
-        record.value[self.name + '.rec_name'] = rec_name
+        record.value.setdefault(self.name + '.', {})['rec_name'] = rec_name
         super(M2OField, self).set_client(record, value,
             force_change=force_change)
 
     def set(self, record, value):
-        rec_name = record.value.get(self.name + '.rec_name') or ''
+        rec_name = record.value.get(self.name + '.', {}).get('rec_name') or ''
         if not rec_name and value is not None and value >= 0:
             try:
                 result, = RPCExecute('model', self.attrs['relation'], 'read',
@@ -455,7 +499,7 @@ class M2OField(Field):
             except RPCException:
                 return
             rec_name = result['rec_name'] or ''
-        record.value[self.name + '.rec_name'] = rec_name
+        record.value.setdefault(self.name + '.', {})['rec_name'] = rec_name
         record.value[self.name] = value
 
     def get_context(self, record, record_context=None, local=False):
@@ -637,10 +681,10 @@ class O2MField(Field):
                 group.load_fields(fields)
 
         if mode == 'list ids':
-            for old_record in group:
-                if old_record.id not in value:
-                    group.remove(old_record, remove=True, signal=False)
-            group.load(value, modified=modified)
+            records_to_remove = [r for r in group if r.id not in value]
+            for record_to_remove in records_to_remove:
+                group.remove(record_to_remove, remove=True, signal=False)
+            group.load(value, modified=modified or default)
         else:
             for vals in value:
                 new_record = record.value[self.name].new(default=False)
@@ -719,37 +763,56 @@ class O2MField(Field):
                 for f in v if f not in fields and f != 'id' and '.' not in f)
             if field_names:
                 try:
-                    fields = RPCExecute('model', self.attrs['relation'],
+                    new_fields = RPCExecute('model', self.attrs['relation'],
                         'fields_get', list(field_names), context=context)
                 except RPCException:
                     return
             else:
-                fields = {}
+                new_fields = {}
 
-        to_remove = []
-        for record2 in record.value[self.name]:
-            if not record2.id:
-                to_remove.append(record2)
+        group = record.value[self.name]
+        if value and value.get('delete'):
+            for record_id in value['delete']:
+                record2 = group.get(record_id)
+                if record2 is not None:
+                    group.remove(
+                        record2, remove=False, signal=False,
+                        force_remove=False)
         if value and value.get('remove'):
             for record_id in value['remove']:
-                record2 = record.value[self.name].get(record_id)
+                record2 = group.get(record_id)
                 if record2 is not None:
-                    to_remove.append(record2)
-        for record2 in to_remove:
-            record.value[self.name].remove(record2, signal=False,
-                force_remove=False)
+                    group.remove(
+                        record2, remove=True, signal=False,
+                        force_remove=False)
 
         if value and (value.get('add') or value.get('update', [])):
-            record.value[self.name].add_fields(fields)
+            # First set already added fields to prevent triggering a
+            # second on_change call
+            for vals in value.get('update', []):
+                if 'id' not in vals:
+                    continue
+                record2 = group.get(vals['id'])
+                if record2 is not None:
+                    vals_to_set = {
+                        k: v for k, v in vals.items() if k not in new_fields}
+                    record2.set_on_change(vals_to_set)
+
+            record.value[self.name].add_fields(new_fields)
             for index, vals in value.get('add', []):
-                new_record = record.value[self.name].new(default=False)
-                record.value[self.name].add(new_record, index, signal=False)
+                new_record = None
+                id_ = vals.pop('id', None)
+                if id_ is not None:
+                    new_record = group.get(id_)
+                if not new_record:
+                    new_record = group.new(obj_id=id_, default=False)
+                group.add(new_record, index, signal=False)
                 new_record.set_on_change(vals)
 
             for vals in value.get('update', []):
                 if 'id' not in vals:
                     continue
-                record2 = record.value[self.name].get(vals['id'])
+                record2 = group.get(vals['id'])
                 if record2 is not None:
                     record2.set_on_change(vals)
 
@@ -814,7 +877,7 @@ class ReferenceField(Field):
     def get_client(self, record):
         if record.value.get(self.name):
             model, _ = record.value[self.name]
-            name = record.value.get(self.name + '.rec_name') or ''
+            name = record.value.get(self.name + '.', {}).get('rec_name') or ''
             return model, name
         else:
             return None
@@ -841,10 +904,11 @@ class ReferenceField(Field):
                     except ValueError:
                         pass
                 if '%s,%s' % (ref_model, ref_id) == self.get(record):
-                    rec_name = record.value.get(self.name + '.rec_name', '')
+                    rec_name = record.value.get(
+                        self.name + '.', {}).get('rec_name') or ''
                 else:
                     rec_name = ''
-            record.value[self.name + '.rec_name'] = rec_name
+            record.value.setdefault(self.name + '.', {})['rec_name'] = rec_name
             value = (ref_model, ref_id)
         super(ReferenceField, self).set_client(record, value,
             force_change=force_change)
@@ -864,7 +928,7 @@ class ReferenceField(Field):
                     pass
         else:
             ref_model, ref_id = value
-        rec_name = record.value.get(self.name + '.rec_name') or ''
+        rec_name = record.value.get(self.name + '.', {}).get('rec_name') or ''
         if ref_model and ref_id is not None and ref_id >= 0:
             if not rec_name and ref_id >= 0:
                 try:
@@ -876,9 +940,9 @@ class ReferenceField(Field):
         elif ref_model:
             rec_name = ''
         else:
-            rec_name = str(ref_id)
+            rec_name = str(ref_id) if ref_id is not None else ''
         record.value[self.name] = ref_model, ref_id
-        record.value[self.name + '.rec_name'] = rec_name
+        record.value.setdefault(self.name + '.', {})['rec_name'] = rec_name
 
     def get_context(self, record, record_context=None, local=False):
         context = super(ReferenceField, self).get_context(
@@ -904,13 +968,14 @@ class ReferenceField(Field):
         else:
             model = None
         screen_domain, attr_domain = self.domains_get(record)
+        screen_domain = filter_leaf(screen_domain, self.name, model)
         screen_domain = prepare_reference_domain(screen_domain, self.name)
         return concat(localize_domain(
-                filter_leaf(screen_domain, self.name, model),
-                strip_target=True), attr_domain)
+                screen_domain, self.name, strip_target=True), attr_domain)
 
     def get_models(self, record):
         screen_domain, attr_domain = self.domains_get(record)
+        screen_domain = prepare_reference_domain(screen_domain, self.name)
         return extract_reference_models(
             concat(screen_domain, attr_domain), self.name)
 
@@ -919,6 +984,12 @@ class _FileCache(object):
     def __init__(self, path):
         self.path = path
 
+    def __del__(self):
+        try:
+            os.remove(self.path)
+        except IOError:
+            pass
+
 
 class BinaryField(Field):
 
@@ -1085,6 +1156,7 @@ TYPES = {
     'one2many': O2MField,
     'reference': ReferenceField,
     'selection': SelectionField,
+    'multiselection': MultiSelectionField,
     'boolean': BooleanField,
     'datetime': DateTimeField,
     'date': DateField,
