@@ -1,7 +1,10 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
-from record import Record
-from field import Field, M2OField, ReferenceField
+import operator
+
+from .record import Record
+from .field import Field, M2OField, ReferenceField
+from tryton import rpc
 from tryton.signal_event import SignalEvent
 from tryton.common.domain_inversion import is_leaf
 from tryton.common import RPCExecute, RPCException, MODELACCESS
@@ -43,7 +46,7 @@ class Group(SignalEvent, list):
     @property
     def readonly(self):
         # Must skip res.user for Preference windows
-        if (self._context.get('_datetime')
+        if (self.context.get('_datetime')
                 or (not (MODELACCESS[self.model_name]['write']
                         or MODELACCESS[self.model_name]['create'])
                     and not self.skip_model_access)):
@@ -90,6 +93,7 @@ class Group(SignalEvent, list):
 
     def insert(self, pos, record):
         assert record.group is self
+        pos = min(pos, len(self))
         if pos >= 1:
             self.__getitem__(pos - 1).next[id(self)] = record
         if pos < self.__len__():
@@ -99,7 +103,7 @@ class Group(SignalEvent, list):
         super(Group, self).insert(pos, record)
         self.__id2record[record.id] = record
         if not self.lock_signal:
-            self.signal('group-list-changed', ('record-added', record))
+            self.signal('group-list-changed', ('record-added', record, pos))
 
     def append(self, record):
         assert record.group is self
@@ -109,7 +113,8 @@ class Group(SignalEvent, list):
         super(Group, self).append(record)
         self.__id2record[record.id] = record
         if not self.lock_signal:
-            self.signal('group-list-changed', ('record-added', record))
+            self.signal('group-list-changed', (
+                    'record-added', record, self.__len__() - 1))
 
     def _remove(self, record):
         idx = self.index(record)
@@ -119,17 +124,22 @@ class Group(SignalEvent, list):
                     self.__getitem__(idx + 1)
             else:
                 self.__getitem__(idx - 1).next[id(self)] = None
-        self.signal('group-list-changed', ('record-removed', record))
+        self.signal('group-list-changed', ('record-removed', record, idx))
         super(Group, self).remove(record)
         del self.__id2record[record.id]
 
     def clear(self):
         # Use reversed order to minimize the cursor reposition as the cursor
         # has more chances to be on top of the list.
+        length = self.__len__()
         for record in reversed(self[:]):
-            self.signal('group-list-changed', ('record-removed', record))
+            # Destroy record before propagating the signal to recursively
+            # destroy also the underlying records
             record.destroy()
+            self.signal(
+                'group-list-changed', ('record-removed', record, length - 1))
             self.pop()
+            length -= 1
         self.__id2record = {}
         self.record_removed, self.record_deleted = [], []
 
@@ -153,7 +163,7 @@ class Group(SignalEvent, list):
         return '<Group %s at %s>' % (self.model_name, id(self))
 
     def load_fields(self, fields):
-        for name, attr in fields.iteritems():
+        for name, attr in fields.items():
             field = Field.get_field(attr['type'])
             attr['name'] = name
             self.fields[name] = field(attr)
@@ -171,7 +181,30 @@ class Group(SignalEvent, list):
         return saved
 
     def delete(self, records):
-        return Record.delete(records)
+        if not records:
+            return
+        root_group = self.root_group
+        assert all(r.model_name == self.model_name for r in records)
+        assert all(r.group.root_group == root_group for r in records)
+        records = [r for r in records if r.id >= 0]
+        ctx = self.context
+        ctx['_timestamp'] = {}
+        for rec in records:
+            ctx['_timestamp'].update(rec.get_timestamp())
+        record_ids = set(r.id for r in records)
+        reload_ids = set(root_group.on_write_ids(list(record_ids)))
+        reload_ids -= record_ids
+        reload_ids = list(reload_ids)
+        try:
+            RPCExecute('model', self.model_name, 'delete', list(record_ids),
+                context=ctx)
+        except RPCException:
+            return False
+        for rec in records:
+            rec.destroy()
+        if reload_ids:
+            root_group.reload(reload_ids)
+        return True
 
     @property
     def root_group(self):
@@ -183,7 +216,7 @@ class Group(SignalEvent, list):
         return root
 
     def written(self, ids):
-        if isinstance(ids, (int, long)):
+        if isinstance(ids, int):
             ids = [ids]
         ids = [x for x in self.on_write_ids(ids) or [] if x not in ids]
         if not ids:
@@ -211,7 +244,7 @@ class Group(SignalEvent, list):
                 return []
         return list({}.fromkeys(res))
 
-    def load(self, ids, modified=False):
+    def load(self, ids, modified=False, position=-1):
         if not ids:
             return True
 
@@ -223,7 +256,11 @@ class Group(SignalEvent, list):
             new_record = self.get(id)
             if not new_record:
                 new_record = Record(self.model_name, id, group=self)
-                self.append(new_record)
+                if position == -1:
+                    self.append(new_record)
+                else:
+                    self.insert(position, new_record)
+                    position += 1
                 new_record.signal_connect(self, 'record-changed',
                     self._record_changed)
                 new_record.signal_connect(self, 'record-modified',
@@ -253,12 +290,24 @@ class Group(SignalEvent, list):
 
     @property
     def context(self):
-        ctx = self._context.copy()
+        return self._get_context(local=False)
+
+    @property
+    def local_context(self):
+        return self._get_context(local=True)
+
+    def _get_context(self, local=False):
+        if not local:
+            ctx = rpc.CONTEXT.copy()
+        else:
+            ctx = {}
         if self.parent:
-            ctx.update(self.parent.get_context())
+            parent_context = self.parent.get_context(local=local)
+            ctx.update(parent_context)
             if self.child_name in self.parent.group.fields:
                 field = self.parent.group.fields[self.child_name]
-                ctx.update(field.get_context(self.parent))
+                ctx.update(field.get_context(
+                        self.parent, parent_context, local=local))
         ctx.update(self._context)
         if self.parent_datetime_field:
             ctx['_datetime'] = self.parent.get_eval(
@@ -276,16 +325,17 @@ class Group(SignalEvent, list):
             record.signal_connect(self, 'record-changed', self._record_changed)
             record.signal_connect(self, 'record-modified',
                 self._record_modified)
-        if position == -1:
-            self.append(record)
-        else:
-            self.insert(position, record)
+        if record not in self:
+            if position == -1:
+                self.append(record)
+            else:
+                self.insert(position, record)
         for record_rm in self.record_removed:
             if record_rm.id == record.id:
-                self.record_removed.remove(record)
+                self.record_removed.remove(record_rm)
         for record_del in self.record_deleted:
             if record_del.id == record.id:
-                self.record_deleted.remove(record)
+                self.record_deleted.remove(record_del)
         self.current_idx = position
         record.modified_fields.setdefault('id')
         record.signal('record-modified')
@@ -301,9 +351,13 @@ class Group(SignalEvent, list):
                     field.set_client(record, value)
         return record
 
-    def set_sequence(self, field='sequence'):
+    def set_sequence(self, field='sequence', position=-1):
         changed = False
         prev = None
+        if position == 0:
+            cmp = operator.gt
+        else:
+            cmp = operator.lt
         for record in self:
             # Assume not loaded records are correctly ordered
             # as far as we do not change any previous records.
@@ -317,11 +371,17 @@ class Group(SignalEvent, list):
                 if value is None:
                     if index:
                         update = True
-                    elif prev and record.id >= 0:
-                        update = record.id < prev.id
+                    elif prev:
+                        if record.id >= 0:
+                            update = cmp(record.id, prev.id)
+                        elif position == 0:
+                            update = True
                 elif value == index:
-                    if prev and record.id >= 0:
-                        update = record.id < prev.id
+                    if prev:
+                        if record.id >= 0:
+                            update = cmp(record.id, prev.id)
+                        elif position == 0:
+                            update = True
                 elif value <= (index or 0):
                     update = True
                 if update:
@@ -358,23 +418,25 @@ class Group(SignalEvent, list):
     def remove(self, record, remove=False, modified=True, signal=True,
             force_remove=False):
         idx = self.index(record)
-        if self[idx].id >= 0:
+        if record.id >= 0:
             if remove:
-                if self[idx] in self.record_deleted:
-                    self.record_deleted.remove(self[idx])
-                self.record_removed.append(self[idx])
+                if record in self.record_deleted:
+                    self.record_deleted.remove(record)
+                if record not in self.record_removed:
+                    self.record_removed.append(record)
             else:
-                if self[idx] in self.record_removed:
-                    self.record_removed.remove(self[idx])
-                self.record_deleted.append(self[idx])
+                if record in self.record_removed:
+                    self.record_removed.remove(record)
+                if record not in self.record_deleted:
+                    self.record_deleted.append(record)
         if record.parent:
             record.parent.modified_fields.setdefault('id')
             record.parent.signal('record-modified')
         if modified:
             record.modified_fields.setdefault('id')
             record.signal('record-modified')
-        if self[idx].id < 0 or force_remove:
-            self._remove(self[idx])
+        if record.id < 0 or force_remove:
+            self._remove(record)
 
         if len(self):
             self.current_idx = min(idx, len(self) - 1)
@@ -399,7 +461,7 @@ class Group(SignalEvent, list):
             return None
         return self[self.current_idx]
 
-    def next(self):
+    def __next__(self):
         if len(self) and self.current_idx is not None:
             self.current_idx = (self.current_idx + 1) % len(self)
         elif len(self):
@@ -410,7 +472,7 @@ class Group(SignalEvent, list):
 
     def add_fields(self, fields):
         to_add = {}
-        for name, attr in fields.iteritems():
+        for name, attr in fields.items():
             if name not in self.fields:
                 to_add[name] = attr
             else:
@@ -428,7 +490,7 @@ class Group(SignalEvent, list):
         if len(new) and len(to_add):
             try:
                 values = RPCExecute('model', self.model_name, 'default_get',
-                    to_add.keys(), context=self.context)
+                    list(to_add.keys()), context=self.context)
             except RPCException:
                 return False
             for record in new:
