@@ -1,9 +1,9 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
-import gtk
-import gobject
 import gettext
 
+from gi.repository import Gdk, GLib, Gtk
+
 from .widget import Widget
 import tryton.common as common
 from tryton.gui.window.view_form.screen import Screen
@@ -13,7 +13,6 @@ from tryton.common.popup_menu import populate
 from tryton.common.completion import get_completion, update_completion
 from tryton.common.entry_position import reset_position
 from tryton.common.domain_parser import quote
-from tryton.config import CONFIG
 
 _ = gettext.gettext
 
@@ -23,11 +22,11 @@ class Many2One(Widget):
     def __init__(self, view, attrs):
         super(Many2One, self).__init__(view, attrs)
 
-        self.widget = gtk.HBox(spacing=0)
+        self.widget = Gtk.HBox(spacing=0)
         self.widget.set_property('sensitive', True)
 
-        self.wid_text = self.mnemonic_widget = gtk.Entry()
-        self.wid_text.set_property('width-chars', 13)
+        self.wid_text = self.mnemonic_widget = Gtk.Entry()
+        self.wid_text.set_property('width-chars', self.default_width_chars)
         self.wid_text.set_property('activates_default', True)
         self.wid_text.connect('key-press-event', self.send_modified)
         self.wid_text.connect('key_press_event', self.sig_key_press)
@@ -44,7 +43,7 @@ class Many2One(Widget):
 
         self.wid_text.connect('icon-press', self.sig_edit)
 
-        self.widget.pack_end(self.wid_text, expand=True, fill=True)
+        self.widget.pack_end(self.wid_text, expand=True, fill=True, padding=0)
 
         self._readonly = False
 
@@ -54,17 +53,13 @@ class Many2One(Widget):
     def _readonly_set(self, value):
         self._readonly = value
         self._set_button_sensitive()
-        if value and CONFIG['client.fast_tabbing']:
-            self.widget.set_focus_chain([])
-        else:
-            self.widget.unset_focus_chain()
 
     def _set_button_sensitive(self):
         self.wid_text.set_editable(not self._readonly)
         self.wid_text.set_icon_sensitive(
-            gtk.ENTRY_ICON_PRIMARY, self.read_access)
+            Gtk.EntryIconPosition.PRIMARY, self.read_access)
         self.wid_text.set_icon_sensitive(
-            gtk.ENTRY_ICON_SECONDARY, not self._readonly)
+            Gtk.EntryIconPosition.SECONDARY, not self._readonly)
 
     def get_access(self, type_):
         model = self.get_model()
@@ -128,7 +123,7 @@ class Many2One(Widget):
                         self.field.set_client(self.record,
                             self.value_from_id(*result[0]), force_change=True)
                     else:
-                        self.set_text('')
+                        self.wid_text.set_text('')
                     self.focus_out = True
                     self.changed = True
 
@@ -137,10 +132,11 @@ class Many2One(Widget):
                     view_ids=self.attrs.get('view_ids', '').split(','),
                     views_preload=self.attrs.get('views', {}),
                     new=self.create_access,
-                    title=self.attrs.get('string'))
+                    title=self.attrs.get('string'),
+                    exclude_field=self.attrs.get('relation_field'))
                 win.screen.search_filter(quote(text))
                 if len(win.screen.group) == 1:
-                    win.response(None, gtk.RESPONSE_OK)
+                    win.response(None, Gtk.ResponseType.OK)
                 else:
                     win.show()
                 return
@@ -148,22 +144,30 @@ class Many2One(Widget):
         self.changed = True
         return
 
-    def get_screen(self):
+    def get_screen(self, search=False):
         domain = self.field.domain_get(self.record)
-        context = self.field.get_context(self.record)
+        if search:
+            context = self.field.get_search_context(self.record)
+        else:
+            context = self.field.get_context(self.record)
         # Remove first tree view as mode is form only
         view_ids = self.attrs.get('view_ids', '').split(',')[1:]
-        return Screen(self.get_model(), domain=domain, context=context,
+        model = self.get_model()
+        breadcrumb = list(self.view.screen.breadcrumb)
+        breadcrumb.append(
+            self.attrs.get('string') or common.MODELNAME.get(model))
+        return Screen(model, domain=domain, context=context,
             mode=['form'], view_ids=view_ids,
             views_preload=self.attrs.get('views', {}), readonly=self._readonly,
-            exclude_field=self.attrs.get('relation_field'))
+            exclude_field=self.attrs.get('relation_field'),
+            breadcrumb=breadcrumb)
 
     def sig_new(self, *args):
         model = self.get_model()
         if not model or not common.MODELACCESS[model]['create']:
             return
         self.focus_out = False
-        screen = self.get_screen()
+        screen = self.get_screen(search=True)
 
         def callback(result):
             if result:
@@ -172,7 +176,7 @@ class Many2One(Widget):
                         screen.current_record.rec_name()))
             self.focus_out = True
         WinForm(screen, callback, new=True, save_current=True,
-            title=self.attrs.get('string'), rec_name=self.wid_text.get_text())
+            rec_name=self.wid_text.get_text())
 
     def sig_edit(self, entry=None, icon_pos=None, *args):
         model = self.get_model()
@@ -184,7 +188,7 @@ class Many2One(Widget):
         self.focus_out = False
         value = self.field.get(self.record)
 
-        if (icon_pos == gtk.ENTRY_ICON_SECONDARY
+        if (icon_pos == Gtk.EntryIconPosition.SECONDARY
                 and not self._readonly
                 and self.has_target(value)):
             self.field.set_client(self.record, self.value_from_id(None, ''))
@@ -205,8 +209,7 @@ class Many2One(Widget):
                         force_change=True)
                 self.focus_out = True
                 self.changed = True
-            WinForm(screen, callback, save_current=True,
-                title=self.attrs.get('string'))
+            WinForm(screen, callback, save_current=True)
             return
         if not self._readonly:
             domain = self.field.domain_get(self.record)
@@ -224,7 +227,8 @@ class Many2One(Widget):
                 context=context, domain=domain, order=order,
                 view_ids=self.attrs.get('view_ids', '').split(','),
                 views_preload=self.attrs.get('views', {}),
-                new=self.create_access, title=self.attrs.get('string'))
+                new=self.create_access, title=self.attrs.get('string'),
+                exclude_field=self.attrs.get('relation_field'))
             win.screen.search_filter(quote(text))
             win.show()
             return
@@ -233,15 +237,15 @@ class Many2One(Widget):
 
     def sig_key_press(self, widget, event, *args):
         editable = self.wid_text.get_editable()
-        activate_keys = [gtk.keysyms.Tab, gtk.keysyms.ISO_Left_Tab]
+        activate_keys = [Gdk.KEY_Tab, Gdk.KEY_ISO_Left_Tab]
         if not self.wid_completion:
-            activate_keys.append(gtk.keysyms.Return)
-        if (event.keyval == gtk.keysyms.F3
+            activate_keys.append(Gdk.KEY_Return)
+        if (event.keyval == Gdk.KEY_F3
                 and editable
                 and self.create_access):
             self.sig_new(widget, event)
             return True
-        elif event.keyval == gtk.keysyms.F2 and self.read_access:
+        elif event.keyval == Gdk.KEY_F2 and self.read_access:
             self.sig_edit(widget)
             return True
         elif (event.keyval in activate_keys
@@ -249,9 +253,8 @@ class Many2One(Widget):
             self.sig_activate()
         elif (self.has_target(self.field.get(self.record))
                 and editable
-                and event.keyval in (gtk.keysyms.Delete,
-                    gtk.keysyms.BackSpace)):
-            self.set_text('')
+                and event.keyval in [Gdk.KEY_Delete, Gdk.KEY_BackSpace]):
+            self.wid_text.set_text('')
         return False
 
     def sig_changed(self, *args):
@@ -270,18 +273,18 @@ class Many2One(Widget):
                 # in such case, the original text should not be restored
                 if not self.wid_text.get_text():
                     # Restore text and position after display
-                    self.set_text(text)
+                    self.wid_text.set_text(text)
                     self.wid_text.set_position(position)
-            gobject.idle_add(clean)
+            GLib.idle_add(clean)
         return False
 
     def get_value(self):
         return self.wid_text.get_text()
 
-    def set_value(self, record, field):
-        if field.get_client(record) != self.wid_text.get_text():
-            field.set_client(record, self.value_from_id(None, ''))
-            self.set_text('')
+    def set_value(self):
+        if self.field.get_client(self.record) != self.wid_text.get_text():
+            self.field.set_client(self.record, self.value_from_id(None, ''))
+            self.wid_text.set_text('')
 
     def set_text(self, value):
         if not value:
@@ -289,19 +292,19 @@ class Many2One(Widget):
         self.wid_text.set_text(value)
         reset_position(self.wid_text)
 
-    def display(self, record, field):
+    def display(self):
         self.changed = False
-        super(Many2One, self).display(record, field)
+        super(Many2One, self).display()
 
         self._set_button_sensitive()
         self._set_completion()
 
-        if not field:
-            self.set_text('')
+        if not self.field:
+            self.set_text(None)
             self.changed = True
             return False
-        self.set_text(field.get_client(record))
-        if self.has_target(field.get(record)):
+        self.set_text(self.field.get_client(self.record))
+        if self.has_target(self.field.get(self.record)):
             icon1, tooltip1 = 'tryton-open', _('Open the record <F2>')
             icon2, tooltip2 = 'tryton-clear', _('Clear the field <Del>')
         else:
@@ -310,11 +313,11 @@ class Many2One(Widget):
         if not self.wid_text.get_editable():
             icon2, tooltip2 = None, ''
         for pos, icon, tooltip in [
-                (gtk.ENTRY_ICON_PRIMARY, icon1, tooltip1),
-                (gtk.ENTRY_ICON_SECONDARY, icon2, tooltip2)]:
+                (Gtk.EntryIconPosition.PRIMARY, icon1, tooltip1),
+                (Gtk.EntryIconPosition.SECONDARY, icon2, tooltip2)]:
             if icon:
                 pixbuf = common.IconFactory.get_pixbuf(
-                    icon, gtk.ICON_SIZE_MENU)
+                    icon, Gtk.IconSize.MENU)
             else:
                 pixbuf = None
             self.wid_text.set_icon_from_pixbuf(pos, pixbuf)
@@ -324,9 +327,9 @@ class Many2One(Widget):
     def _populate_popup(self, widget, menu):
         value = self.field.get(self.record)
         if self.has_target(value):
-            # Delay filling of popup as it can take time
-            gobject.idle_add(populate, menu, self.get_model(),
-                self.id_from_value(value), '', self.field)
+            populate(
+                menu, self.get_model(), self.id_from_value(value),
+                '', self.field, self.field.get_context(self.record))
         return True
 
     def _set_completion(self):
@@ -345,7 +348,7 @@ class Many2One(Widget):
         rec_name, record_id = model.get(iter_, 0, 1)
         # GTK on win32 doesn't like synchronous call to set_client
         # because it triggers a display which reset the completion
-        gobject.idle_add(self.field.set_client, self.record,
+        GLib.idle_add(self.field.set_client, self.record,
             self.value_from_id(record_id, rec_name), True)
 
         completion_model = self.wid_completion.get_model()
