@@ -1,9 +1,10 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
-import gtk
 import gettext
 import itertools
 
+from gi.repository import Gdk, Gtk
+
 from .widget import Widget
 from tryton.gui.window.view_form.screen import Screen
 from tryton.gui.window.win_search import WinSearch
@@ -22,66 +23,66 @@ class One2Many(Widget):
     def __init__(self, view, attrs):
         super(One2Many, self).__init__(view, attrs)
 
-        self.widget = gtk.Frame()
-        self.widget.set_shadow_type(gtk.SHADOW_NONE)
+        self.widget = Gtk.Frame()
+        self.widget.set_shadow_type(Gtk.ShadowType.NONE)
         self.widget.get_accessible().set_name(attrs.get('string', ''))
-        vbox = gtk.VBox(homogeneous=False, spacing=2)
+        vbox = Gtk.VBox(homogeneous=False, spacing=2)
         self.widget.add(vbox)
         self._readonly = True
         self._required = False
         self._position = 0
         self._length = 0
 
-        self.title_box = hbox = gtk.HBox(homogeneous=False, spacing=0)
+        self.title_box = hbox = Gtk.HBox(homogeneous=False, spacing=0)
         hbox.set_border_width(2)
 
-        self.title = gtk.Label(set_underline(attrs.get('string', '')))
-        self.title.set_use_underline(True)
-        self.title.set_alignment(0.0, 0.5)
-        hbox.pack_start(self.title, expand=True, fill=True)
+        self.title = Gtk.Label(
+            label=set_underline(attrs.get('string', '')),
+            use_underline=True, halign=Gtk.Align.START)
+        hbox.pack_start(self.title, expand=True, fill=True, padding=0)
 
-        hbox.pack_start(gtk.VSeparator(), expand=False, fill=True)
+        hbox.pack_start(Gtk.VSeparator(), expand=False, fill=True, padding=0)
 
         tooltips = common.Tooltips()
 
-        but_switch = gtk.Button()
+        but_switch = Gtk.Button(can_focus=False)
         tooltips.set_tip(but_switch, _('Switch'))
         but_switch.connect('clicked', self.switch_view)
         but_switch.add(common.IconFactory.get_image(
-                'tryton-switch', gtk.ICON_SIZE_SMALL_TOOLBAR))
-        but_switch.set_relief(gtk.RELIEF_NONE)
-        hbox.pack_start(but_switch, expand=False, fill=False)
+                'tryton-switch', Gtk.IconSize.SMALL_TOOLBAR))
+        but_switch.set_relief(Gtk.ReliefStyle.NONE)
+        hbox.pack_start(but_switch, expand=False, fill=False, padding=0)
 
-        self.but_pre = gtk.Button()
+        self.but_pre = Gtk.Button(can_focus=False)
         tooltips.set_tip(self.but_pre, _('Previous'))
         self.but_pre.connect('clicked', self._sig_previous)
         self.but_pre.add(common.IconFactory.get_image(
-                'tryton-back', gtk.ICON_SIZE_SMALL_TOOLBAR))
-        self.but_pre.set_relief(gtk.RELIEF_NONE)
-        hbox.pack_start(self.but_pre, expand=False, fill=False)
+                'tryton-back', Gtk.IconSize.SMALL_TOOLBAR))
+        self.but_pre.set_relief(Gtk.ReliefStyle.NONE)
+        hbox.pack_start(self.but_pre, expand=False, fill=False, padding=0)
 
-        self.label = gtk.Label('(0,0)')
-        hbox.pack_start(self.label, expand=False, fill=False)
+        self.label = Gtk.Label(label='(0,0)')
+        hbox.pack_start(self.label, expand=False, fill=False, padding=0)
 
-        self.but_next = gtk.Button()
+        self.but_next = Gtk.Button(can_focus=False)
         tooltips.set_tip(self.but_next, _('Next'))
         self.but_next.connect('clicked', self._sig_next)
         self.but_next.add(common.IconFactory.get_image(
-                'tryton-forward', gtk.ICON_SIZE_SMALL_TOOLBAR))
-        self.but_next.set_relief(gtk.RELIEF_NONE)
-        hbox.pack_start(self.but_next, expand=False, fill=False)
+                'tryton-forward', Gtk.IconSize.SMALL_TOOLBAR))
+        self.but_next.set_relief(Gtk.ReliefStyle.NONE)
+        hbox.pack_start(self.but_next, expand=False, fill=False, padding=0)
 
-        hbox.pack_start(gtk.VSeparator(), expand=False, fill=True)
+        hbox.pack_start(Gtk.VSeparator(), expand=False, fill=True, padding=0)
 
         self.focus_out = True
         self.wid_completion = None
         if attrs.get('add_remove'):
 
-            self.wid_text = gtk.Entry()
+            self.wid_text = Gtk.Entry()
             self.wid_text.set_placeholder_text(_('Search'))
             self.wid_text.set_property('width_chars', 13)
             self.wid_text.connect('focus-out-event', self._focus_out)
-            hbox.pack_start(self.wid_text, expand=True, fill=True)
+            hbox.pack_start(self.wid_text, expand=True, fill=True, padding=0)
 
             if int(self.attrs.get('completion', 1)):
                 access = common.MODELACCESS[attrs['relation']]
@@ -95,80 +96,83 @@ class One2Many(Widget):
                 self.wid_text.set_completion(self.wid_completion)
                 self.wid_text.connect('changed', self._update_completion)
 
-            self.but_add = gtk.Button()
+            self.but_add = Gtk.Button(can_focus=False)
             tooltips.set_tip(self.but_add, _('Add existing record'))
             self.but_add.connect('clicked', self._sig_add)
             self.but_add.add(common.IconFactory.get_image(
-                    'tryton-add', gtk.ICON_SIZE_SMALL_TOOLBAR))
-            self.but_add.set_relief(gtk.RELIEF_NONE)
-            hbox.pack_start(self.but_add, expand=False, fill=False)
+                    'tryton-add', Gtk.IconSize.SMALL_TOOLBAR))
+            self.but_add.set_relief(Gtk.ReliefStyle.NONE)
+            hbox.pack_start(self.but_add, expand=False, fill=False, padding=0)
 
-            self.but_remove = gtk.Button()
+            self.but_remove = Gtk.Button(can_focus=False)
             tooltips.set_tip(self.but_remove,
                 _('Remove selected record'))
             self.but_remove.connect('clicked', self._sig_remove, True)
             self.but_remove.add(common.IconFactory.get_image(
-                    'tryton-remove', gtk.ICON_SIZE_SMALL_TOOLBAR))
-            self.but_remove.set_relief(gtk.RELIEF_NONE)
-            hbox.pack_start(self.but_remove, expand=False, fill=False)
+                    'tryton-remove', Gtk.IconSize.SMALL_TOOLBAR))
+            self.but_remove.set_relief(Gtk.ReliefStyle.NONE)
+            hbox.pack_start(
+                self.but_remove, expand=False, fill=False, padding=0)
 
-            hbox.pack_start(gtk.VSeparator(), expand=False, fill=True)
+            hbox.pack_start(
+                Gtk.VSeparator(), expand=False, fill=True, padding=0)
 
-        self.but_new = gtk.Button()
-        tooltips.set_tip(self.but_new, _('Create a new record <F3>'))
+        self.but_new = Gtk.Button(can_focus=False)
+        tooltips.set_tip(self.but_new, _('Create a new record'))
         self.but_new.connect('clicked', self._sig_new)
         self.but_new.add(common.IconFactory.get_image(
-                'tryton-create', gtk.ICON_SIZE_SMALL_TOOLBAR))
-        self.but_new.set_relief(gtk.RELIEF_NONE)
-        hbox.pack_start(self.but_new, expand=False, fill=False)
+                'tryton-create', Gtk.IconSize.SMALL_TOOLBAR))
+        self.but_new.set_relief(Gtk.ReliefStyle.NONE)
+        hbox.pack_start(self.but_new, expand=False, fill=False, padding=0)
 
-        self.but_open = gtk.Button()
-        tooltips.set_tip(self.but_open, _('Edit selected record <F2>'))
+        self.but_open = Gtk.Button(can_focus=False)
+        tooltips.set_tip(self.but_open, _('Edit selected record'))
         self.but_open.connect('clicked', self._sig_edit)
         self.but_open.add(common.IconFactory.get_image(
-                'tryton-open', gtk.ICON_SIZE_SMALL_TOOLBAR))
-        self.but_open.set_relief(gtk.RELIEF_NONE)
-        hbox.pack_start(self.but_open, expand=False, fill=False)
+                'tryton-open', Gtk.IconSize.SMALL_TOOLBAR))
+        self.but_open.set_relief(Gtk.ReliefStyle.NONE)
+        hbox.pack_start(self.but_open, expand=False, fill=False, padding=0)
 
-        self.but_del = gtk.Button()
-        tooltips.set_tip(self.but_del, _('Delete selected record <Del>'))
+        self.but_del = Gtk.Button(can_focus=False)
+        tooltips.set_tip(self.but_del, _('Delete selected record'))
         self.but_del.connect('clicked', self._sig_remove, False)
         self.but_del.add(common.IconFactory.get_image(
-                'tryton-delete', gtk.ICON_SIZE_SMALL_TOOLBAR))
-        self.but_del.set_relief(gtk.RELIEF_NONE)
-        hbox.pack_start(self.but_del, expand=False, fill=False)
+                'tryton-delete', Gtk.IconSize.SMALL_TOOLBAR))
+        self.but_del.set_relief(Gtk.ReliefStyle.NONE)
+        hbox.pack_start(self.but_del, expand=False, fill=False, padding=0)
 
-        self.but_undel = gtk.Button()
+        self.but_undel = Gtk.Button(can_focus=False)
         tooltips.set_tip(self.but_undel, _('Undelete selected record <Ins>'))
         self.but_undel.connect('clicked', self._sig_undelete)
         self.but_undel.add(common.IconFactory.get_image(
-                'tryton-undo', gtk.ICON_SIZE_SMALL_TOOLBAR))
-        self.but_undel.set_relief(gtk.RELIEF_NONE)
-        hbox.pack_start(self.but_undel, expand=False, fill=False)
-
-        if attrs.get('add_remove'):
-            hbox.set_focus_chain([self.wid_text])
-        else:
-            hbox.set_focus_chain([])
+                'tryton-undo', Gtk.IconSize.SMALL_TOOLBAR))
+        self.but_undel.set_relief(Gtk.ReliefStyle.NONE)
+        hbox.pack_start(self.but_undel, expand=False, fill=False, padding=0)
 
         tooltips.enable()
 
-        frame = gtk.Frame()
+        frame = Gtk.Frame()
         frame.add(hbox)
-        frame.set_shadow_type(gtk.SHADOW_OUT)
-        vbox.pack_start(frame, expand=False, fill=True)
+        frame.set_shadow_type(Gtk.ShadowType.OUT)
+        vbox.pack_start(frame, expand=False, fill=True, padding=0)
 
-        self.screen = Screen(attrs['relation'],
+        model = attrs['relation']
+        breadcrumb = list(self.view.screen.breadcrumb)
+        breadcrumb.append(
+            attrs.get('string') or common.MODELNAME.get(model))
+        self.screen = Screen(model,
             mode=attrs.get('mode', 'tree,form').split(','),
             view_ids=attrs.get('view_ids', '').split(','),
             views_preload=attrs.get('views', {}),
+            order=attrs.get('order'),
             row_activate=self._on_activate,
             exclude_field=attrs.get('relation_field', None),
-            limit=None)
+            limit=None,
+            breadcrumb=breadcrumb)
         self.screen.pre_validate = bool(int(attrs.get('pre_validate', 0)))
         self.screen.signal_connect(self, 'record-message', self._sig_label)
 
-        vbox.pack_start(self.screen.widget, expand=True, fill=True)
+        vbox.pack_start(self.screen.widget, expand=True, fill=True, padding=0)
 
         self.title.set_mnemonic_widget(
             self.screen.current_view.mnemonic_widget)
@@ -180,27 +184,38 @@ class One2Many(Widget):
         but_switch.props.sensitive = self.screen.number_of_views > 1
 
     def on_keypress(self, widget, event):
-        if (event.keyval == gtk.keysyms.F3) \
-                and self.but_new.get_property('sensitive'):
+        if ((event.keyval == Gdk.KEY_F3)
+                and self.but_new.get_property('sensitive')):
             self._sig_new(widget)
             return True
-        if event.keyval == gtk.keysyms.F2 \
-                and widget == self.screen.widget:
-            self._sig_edit(widget)
-            return True
-        if (event.keyval in (gtk.keysyms.Delete, gtk.keysyms.KP_Delete)
-                and widget == self.screen.widget
-                and self.but_del.get_property('sensitive')):
-            self._sig_remove(widget)
-            return True
-        if event.keyval == gtk.keysyms.Insert and widget == self.screen.widget:
+        if event.keyval == Gdk.KEY_F2:
+            if widget == self.screen.widget:
+                self._sig_edit(widget)
+                return True
+            elif widget == self.wid_text:
+                self._sig_add(widget)
+                return True
+        if event.keyval == Gdk.KEY_F4:
+            self.switch_view(widget)
+        if (event.keyval in [Gdk.KEY_Delete, Gdk.KEY_KP_Delete]
+                and widget == self.screen.widget):
+            remove = not (event.state & Gdk.ModifierType.CONTROL_MASK)
+            if remove and self.attrs.get('add_remove'):
+                but = self.but_remove
+            else:
+                remove = False
+                but = self.but_del
+            if but.get_property('sensitive'):
+                self._sig_remove(widget, remove)
+                return True
+        if event.keyval == Gdk.KEY_Insert and widget == self.screen.widget:
             self._sig_undelete(widget)
             return True
         if self.attrs.get('add_remove'):
             editable = self.wid_text.get_editable()
-            activate_keys = [gtk.keysyms.Tab, gtk.keysyms.ISO_Left_Tab]
+            activate_keys = [Gdk.KEY_Tab, Gdk.KEY_ISO_Left_Tab]
             if not self.wid_completion:
-                activate_keys.append(gtk.keysyms.Return)
+                activate_keys.append(Gdk.KEY_Return)
             if (widget == self.wid_text
                     and event.keyval in activate_keys
                     and editable
@@ -258,6 +273,7 @@ class One2Many(Widget):
         if isinstance(self._position, int):
             first = self._position <= 1
             last = self._position >= self._length
+        deletable = self.screen.deletable
 
         self.but_new.set_sensitive(bool(
                 not self._readonly
@@ -267,6 +283,7 @@ class One2Many(Widget):
         self.but_del.set_sensitive(bool(
                 not self._readonly
                 and self.attrs.get('delete', True)
+                and deletable
                 and self._position
                 and access['delete']))
         self.but_undel.set_sensitive(bool(
@@ -296,17 +313,6 @@ class One2Many(Widget):
             self.wid_text.set_sensitive(self.but_add.get_sensitive())
             self.wid_text.set_editable(self.but_add.get_sensitive())
 
-        # New button must be added to focus chain to allow keyboard only
-        # creation when there is no existing record on form view.
-        focus_chain = self.title_box.get_focus_chain() or []
-        if o2m_size == 0 and self.screen.current_view.view_type == 'form':
-            if self.but_new not in focus_chain:
-                focus_chain.append(self.but_new)
-        else:
-            if self.but_new in focus_chain:
-                focus_chain.remove(self.but_new)
-        self.title_box.set_focus_chain(focus_chain)
-
     def _validate(self):
         self.view.set_value()
         record = self.screen.current_record
@@ -338,13 +344,12 @@ class One2Many(Widget):
             self._new_single()
 
     def _new_single(self):
-        ctx = {}
-        ctx.update(self.field.get_context(self.record))
         sequence = self._sequence()
 
         def update_sequence():
             if sequence:
-                self.screen.group.set_sequence(field=sequence)
+                self.screen.group.set_sequence(
+                    field=sequence, position=self.screen.new_position)
 
         if self.screen.current_view.editable:
             self.screen.new()
@@ -354,7 +359,7 @@ class One2Many(Widget):
             field_size = self.record.expr_eval(self.attrs.get('size')) or -1
             field_size -= len(self.field.get_eval(self.record)) + 1
             WinForm(self.screen, lambda a: update_sequence(), new=True,
-                many=field_size, context=ctx, title=self.attrs.get('string'))
+                many=field_size)
 
     def _new_product(self):
         fields = self.attrs['product'].split(',')
@@ -404,7 +409,8 @@ class One2Many(Widget):
 
             sequence = self._sequence()
             if sequence:
-                self.screen.group.set_sequence(field=sequence)
+                self.screen.group.set_sequence(
+                    field=sequence, position=self.screen.new_position)
 
         search_set()
 
@@ -415,8 +421,7 @@ class One2Many(Widget):
             return
         record = self.screen.current_record
         if record:
-            WinForm(self.screen, lambda a: None,
-                title=self.attrs.get('string'))
+            WinForm(self.screen, lambda a: None)
 
     def _sig_next(self, widget):
         if not self._validate():
@@ -430,11 +435,13 @@ class One2Many(Widget):
 
     def _sig_remove(self, widget, remove=False):
         access = common.MODELACCESS[self.screen.model_name]
+        writable = not self.screen.readonly
+        deletable = self.screen.deletable
         if remove:
-            if not access['write'] or not access['read']:
+            if not access['write'] or not writable or not access['read']:
                 return
         else:
-            if not access['delete']:
+            if not access['delete'] or not deletable:
                 return
         self.screen.remove(remove=remove)
 
@@ -466,7 +473,8 @@ class One2Many(Widget):
                 self.screen.load(ids, modified=True)
                 self.screen.display(res_id=ids[0])
                 if sequence:
-                    self.screen.group.set_sequence(field=sequence)
+                    self.screen.group.set_sequence(
+                        field=sequence, position=self.screen.new_position)
             self.screen.set_cursor()
             self.wid_text.set_text('')
 
@@ -476,7 +484,8 @@ class One2Many(Widget):
             view_ids=self.attrs.get('view_ids', '').split(','),
             views_preload=self.attrs.get('views', {}),
             new=self.but_new.get_property('sensitive'),
-            title=self.attrs.get('string'))
+            title=self.attrs.get('string'),
+            exclude_field=self.attrs.get('relation_field'))
         win.screen.search_filter(quote(text))
         win.show()
 
@@ -491,18 +500,18 @@ class One2Many(Widget):
         self.label.set_text(line)
         self._set_button_sensitive()
 
-    def display(self, record, field):
-        super(One2Many, self).display(record, field)
+    def display(self):
+        super(One2Many, self).display()
 
         self._set_button_sensitive()
 
-        if field is None:
+        if not self.field:
             self.screen.new_group()
             self.screen.current_record = None
             self.screen.parent = None
             self.screen.display()
             return False
-        new_group = field.get_client(record)
+        new_group = self.field.get_client(self.record)
 
         if id(self.screen.group) != id(new_group):
             self.screen.group = new_group
@@ -511,9 +520,9 @@ class One2Many(Widget):
                 self.screen.current_record = None
         domain = []
         size_limit = None
-        if record:
-            domain = field.domain_get(record)
-            size_limit = record.expr_eval(self.attrs.get('size'))
+        if self.record:
+            domain = self.field.domain_get(self.record)
+            size_limit = self.record.expr_eval(self.attrs.get('size'))
         if self._readonly:
             if size_limit is None:
                 size_limit = len(self.screen.group)
@@ -525,11 +534,11 @@ class One2Many(Widget):
         self.screen.display()
         return True
 
-    def set_value(self, record, field):
+    def set_value(self):
         self.screen.current_view.set_value()
         if self.screen.modified():  # TODO check if required
-            record.modified_fields.setdefault(field.name)
-            record.signal('record-modified')
+            self.record.modified_fields.setdefault(self.field.name)
+            self.record.signal('record-modified')
         return True
 
     def _completion_match_selected(self, completion, model, iter_):
