@@ -1,12 +1,10 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
-import gtk
-import gobject
+from gi.repository import GLib, Gtk
 
 from .widget import Widget
 from tryton.common.selection import SelectionMixin, selection_shortcuts, \
     PopdownMixin
-from tryton.config import CONFIG
 
 
 class Selection(Widget, SelectionMixin, PopdownMixin):
@@ -14,12 +12,12 @@ class Selection(Widget, SelectionMixin, PopdownMixin):
     def __init__(self, view, attrs):
         super(Selection, self).__init__(view, attrs)
 
-        self.widget = gtk.HBox(spacing=3)
-        self.entry = gtk.ComboBoxEntry()
+        self.widget = Gtk.HBox(spacing=3)
+        self.entry = Gtk.ComboBox(has_entry=True)
         child = self.mnemonic_widget = self.entry.get_child()
         child.set_property('activates_default', True)
         child.set_max_length(int(attrs.get('size', 0)))
-        child.set_width_chars(10)
+        child.set_width_chars(self.default_width_chars)
 
         selection_shortcuts(self.entry)
         child.connect('activate', lambda *a: self._focus_out())
@@ -27,9 +25,9 @@ class Selection(Widget, SelectionMixin, PopdownMixin):
         self.entry.connect('changed', self.changed)
         self.entry.connect('move-active', self._move_active)
         self.entry.connect(
-            'scroll-event', lambda c, e: c.emit_stop_by_name('scroll-event'))
-        self.widget.pack_start(self.entry)
-        self.widget.set_focus_chain([child])
+            'scroll-event',
+            lambda c, e: c.stop_emission_by_name('scroll-event'))
+        self.widget.pack_start(self.entry, expand=True, fill=True, padding=0)
 
         self.selection = attrs.get('selection', [])[:]
         self.attrs = attrs
@@ -41,21 +39,17 @@ class Selection(Widget, SelectionMixin, PopdownMixin):
             if combobox.props.window:
                 self._focus_out()
         # Must be deferred because it triggers a display of the form
-        gobject.idle_add(focus_out)
+        GLib.idle_add(focus_out)
 
     def _move_active(self, combobox, scroll_type):
         if not combobox.get_child().get_editable():
-            combobox.emit_stop_by_name('move-active')
+            combobox.stop_emission_by_name('move-active')
 
     def _readonly_set(self, value):
         super(Selection, self)._readonly_set(value)
         self.entry.get_child().set_editable(not value)
         self.entry.set_button_sensitivity(
-            gtk.SENSITIVITY_OFF if value else gtk.SENSITIVITY_AUTO)
-        if value and CONFIG['client.fast_tabbing']:
-            self.widget.set_focus_chain([])
-        else:
-            self.widget.unset_focus_chain()
+            Gtk.SensitivityType.OFF if value else Gtk.SensitivityType.AUTO)
 
     def get_value(self):
         if not self.entry.get_child():  # entry is destroyed
@@ -68,22 +62,22 @@ class Selection(Widget, SelectionMixin, PopdownMixin):
             return self.field.get(self.record) != self.get_value()
         return False
 
-    def set_value(self, record, field):
+    def set_value(self):
         value = self.get_value()
         if 'relation' in self.attrs and value:
             value = (value, self.get_popdown_text(self.entry))
-        field.set_client(record, value)
+        self.field.set_client(self.record, value)
 
-    def display(self, record, field):
-        self.update_selection(record, field)
+    def display(self):
+        self.update_selection(self.record, self.field)
         self.set_popdown(self.selection, self.entry)
-        if not field:
+        if not self.field:
             self.entry.set_active(-1)
             # When setting no item GTK doesn't clear the entry
             self.entry.get_child().set_text('')
             return
-        super(Selection, self).display(record, field)
-        value = field.get(record)
+        super(Selection, self).display()
+        value = self.field.get(self.record)
         if isinstance(value, (list, tuple)):
             # Compatibility with Many2One
             value = value[0]
