@@ -11,9 +11,8 @@ except ImportError:
 
 from functools import partial
 
-from tryton import bus
+from tryton import bus, device_cookie, fingerprints
 from tryton.jsonrpc import ServerProxy, ServerPool, Fault
-from tryton.fingerprints import Fingerprints
 from tryton.config import get_config_dir
 from tryton.exceptions import TrytonServerError, TrytonServerUnavailable
 from tryton.config import CONFIG
@@ -27,11 +26,10 @@ _KEYWORD_CACHE = {}
 _CA_CERTS = os.path.join(get_config_dir(), 'ca_certs')
 if not os.path.isfile(_CA_CERTS):
     _CA_CERTS = None
-_FINGERPRINTS = Fingerprints()
 
-ServerProxy = partial(ServerProxy, fingerprints=_FINGERPRINTS,
+ServerProxy = partial(ServerProxy, fingerprints=fingerprints,
     ca_certs=_CA_CERTS)
-ServerPool = partial(ServerPool, fingerprints=_FINGERPRINTS,
+ServerPool = partial(ServerPool, fingerprints=fingerprints,
     ca_certs=_CA_CERTS)
 
 
@@ -74,13 +72,13 @@ def server_version(host, port):
 def login(parameters):
     from tryton import common
     global CONNECTION, _USER
-    global _VIEW_CACHE, _TOOLBAR_CACHE, _KEYWORD_CACHE
     host = CONFIG['login.host']
     hostname = common.get_hostname(host)
     port = common.get_port(host)
     database = CONFIG['login.db']
     username = CONFIG['login.login']
     language = CONFIG['client.lang']
+    parameters['device_cookie'] = device_cookie.get()
     connection = ServerProxy(hostname, port, database)
     logging.getLogger(__name__).info('common.db.login(%s, %s, %s)'
         % (username, 'x' * 10, language))
@@ -90,17 +88,14 @@ def login(parameters):
     session = ':'.join(map(str, [username] + result))
     if CONNECTION is not None:
         CONNECTION.close()
-    CONNECTION = ServerPool(hostname, port, database, session=session)
-    _VIEW_CACHE = {}
-    _TOOLBAR_CACHE = {}
-    _KEYWORD_CACHE = {}
-
+    CONNECTION = ServerPool(
+        hostname, port, database, session=session, cache=not CONFIG['dev'])
+    device_cookie.renew()
     bus.listen(CONNECTION)
 
 
 def logout():
     global CONNECTION, _USER
-    global _VIEW_CACHE, _TOOLBAR_CACHE, _KEYWORD_CACHE
     if CONNECTION is not None:
         try:
             logging.getLogger(__name__).info('common.db.logout()')
@@ -111,31 +106,12 @@ def logout():
         CONNECTION.close()
         CONNECTION = None
     _USER = None
-    _VIEW_CACHE = {}
-    _TOOLBAR_CACHE = {}
-    _KEYWORD_CACHE = {}
 
 
-def _execute(blocking, *args):
+def execute(*args):
     global CONNECTION, _USER
     if CONNECTION is None:
         raise TrytonServerError('403')
-    key = False
-    model = args[1]
-    method = args[2]
-    if not CONFIG['dev']:
-        if method == 'fields_view_get':
-            key = str(args)
-            if key in _VIEW_CACHE:
-                return _VIEW_CACHE[key]
-        elif method == 'view_toolbar_get':
-            key = str(args)
-            if key in _TOOLBAR_CACHE:
-                return _TOOLBAR_CACHE[key]
-        elif model == 'ir.action.keyword' and method == 'get_keyword':
-            key = str(args)
-            if key in _KEYWORD_CACHE:
-                return _KEYWORD_CACHE[key]
     try:
         name = '.'.join(args[:3])
         args = args[3:]
@@ -144,20 +120,10 @@ def _execute(blocking, *args):
             result = getattr(conn, name)(*args)
     except (http.client.CannotSendRequest, socket.error) as exception:
         raise TrytonServerUnavailable(*exception.args)
-    if not CONFIG['dev']:
-        if key and method == 'fields_view_get':
-            _VIEW_CACHE[key] = result
-        elif key and method == 'view_toolbar_get':
-            _TOOLBAR_CACHE[key] = result
-        elif key and model == 'ir.action.keyword' and method == 'get_keyword':
-            _KEYWORD_CACHE[key] = result
     logging.getLogger(__name__).debug(repr(result))
     return result
 
 
-def execute(*args):
-    return _execute(True, *args)
-
-
-def execute_nonblocking(*args):
-    return _execute(False, *args)
+def clear_cache(prefix=None):
+    if CONNECTION:
+        CONNECTION.clear_cache(prefix)
